"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InternalTypeUtil = void 0;
const CompilationError_1 = require("../Util/CompilationError");
class InternalTypeUtil {
}
exports.InternalTypeUtil = InternalTypeUtil;
InternalTypeUtil.TypeString = {
    type: 'string',
};
InternalTypeUtil.TypeNumber = {
    type: 'number',
};
InternalTypeUtil.TypeEnum = {
    type: 'enum',
};
InternalTypeUtil.TypeRegex = {
    type: 'regex',
};
InternalTypeUtil.TypeDate = {
    type: 'date',
};
InternalTypeUtil.TypeBoolean = {
    type: 'boolean',
};
InternalTypeUtil.TypeAnyObject = {
    type: 'object',
};
InternalTypeUtil.TypeSimpleObject = {
    type: 'object',
    assertValid(def, node) {
        const validTypes = ['string', 'number', 'boolean', 'null', 'undefined'];
        if (def.type !== 'object' || typeof def.schema !== 'object') {
            throw new CompilationError_1.CompilationError('Invalid type definition: expected object', node);
        }
        let schema = def.schema;
        if (schema.$ref) {
            const ref = schema.$ref.split('/');
            const def = schema.definitions[ref[ref.length - 1]];
            if (!def) {
                throw new CompilationError_1.CompilationError(`Invalid type definition: schema reference '${schema.$ref}' not found`, node);
            }
            schema = {
                ...schema,
                ...def,
            };
        }
        if (schema?.type !== 'object') {
            throw new CompilationError_1.CompilationError('Invalid type definition: expected object', node);
        }
        for (const [name, property] of Object.entries(schema.properties ?? {})) {
            const types = typeof property.type === 'string' ? [property.type] : property.type;
            const badType = types.find(t => !validTypes.includes(t));
            if (badType) {
                throw new CompilationError_1.CompilationError(`Invalid type definition: unsupported type '${badType}' on object property ${name}. Expected one of ${validTypes.join(', ')}`, node);
            }
        }
    }
};
InternalTypeUtil.TypeAnyArray = {
    type: 'array',
    elementType: {
        type: 'any',
    },
};
InternalTypeUtil.TypeNumberArray = {
    type: 'array',
    elementType: {
        type: 'number',
    },
};
InternalTypeUtil.TypeAnyFunction = {
    type: 'function',
};
InternalTypeUtil.TypeAny = {
    type: 'any',
};
InternalTypeUtil.TypeBuffer = {
    type: 'Buffer',
};
//# sourceMappingURL=InternalTypes.js.map